/*
 *      Conway.c                        1.006                   23.04.2011
 *
 *      Primzahl-Berechnung nach John Horton Conway in optimierter Weise,
 *      gem [1995OliS], Seite 30-38 im Vergleich mit Seite 24-25.
 *      berprfung des Algorithmus gem 1.Thessalonicher 5,21-22.
 *
 *      Literatur:
 *      [0397Bibel]
 *      "Die Bibel, oder die ganze Heilige Schrift Alten und Neuen Testaments
 *      nach der deutschen Uebersetzung Dr. Martin Luthers.", Revision durch
 *      Dr. Johann Philipp Fresenius, (1751); Druck und Verlag von Heinrich
 *      Ludwig Brnner, Frankfurt am Main, 40. Auflage, (1841); der Text gilt
 *      als vollstndig abgeschlossen seit dem Konzil von Karthago, (397)
 *
 *      [1995OliS]
 *      Olivastro D., Schmidt M.: Das chinesische Dreieck (Ancient Puzzles),
 *      Lizenzausgabe mit freundlicher Genehmigung des Droemer Knaur
 *      Verlages, Mnchen fr Zweitausendeins, Frankfurt am Main, (1995)
 *
 *      Bearbeitung:
 *      28.03.2011 - 23.04.2011 Norbert Sdland, D-73431 Aalen
 */

/*
 *      Zugang:
 *      Gegeben seien die 14 Brche (in Primzahl-Darstellung):
 *
 *       17     2*3*13   19      23      29     7*11    5*19
 *                            
 *      7*13     5*17   3*17    2*19    3*11     29      23
 *
 *      7*11     1      11      13      3*5     3*5     55
 *                                      
 *       19     17      13      11      2*7      2       1
 *
 *      Beginnend mit der Zahl 2 wird jeweils der Reihe nach die nchste
 *      natrliche Zahl bestimmt, die sich durch Multiplikation mit dem
 *      entsprechenden Bruch ergibt, dann mit dieser Zahl weiter wieder
 *      beim ersten Bruch beginnend, bis nur noch ein Vielfaches von 2
 *      resultiert. Der Exponent dieser Zweierpotenz ist dann die nchste
 *      Primzahl.
 *
 *      Das Problem ist nun, dass die Rechnung hier geschickt umgestellt
 *      werden muss, um nicht in numerische Probleme zu geraten, denn die
 *      14.Primzahl ist 43, und 2^43 ist eine 12-stellige Zahl. Bei der
 *      Konstruktion der 14 Brche knnen somit mindestens 14 Primzahlen
 *      direkt durch Festlegung von 14 Parametern einprogrammiert werden.
 *
 *      Da der Algorithmus nicht erklrt wird, sondern als etwas besonders
 *      "Geniales" dargestellt wird, ist es sinnvoll, die Arbeitsweise
 *      dieses Primzahlengenerators experimentell zu berprfen.
 *
 *      Dabei stellt sich zunchst heraus, dass der Algorithmus umstndlich
 *      verpackt wurde und durch Betrachtung der einzelnen Primfaktoren
 *      in 10 verschiedenen Registern abgehandelt werden kann, um so auch
 *      noch groe Primzahlen sicher zu bestimmen.
 *
 *      Es ist fr die Arbeitsweise des vorgestellten Algorithmus vllig
 *      egal, welche Primzahlen (oder Buchstaben usw.) fr die Benennung
 *      der 10 Register gewhlt werden. Aus Grnden der bersichtlichkeit
 *      wird hier der historische Weg des John Horton Conway beschritten.
 *      Der Mathematik-Professor Donald E. Knuth, von welchem das
 *      TeX-Programm stammt, soll Conway als "Alpha und Omega" bezeichnet
 *      haben, und auch eine mythische Gestalt "J.H.W.H. Conway" eingefhrt
 *      haben (vgl. [1995]OliS], Anmerkung auf Seite 30), was im gnstigsten
 *      Fall mit den Worten des Apostels Paulus heit: "Wir sind seines
 *      Geschlechts." (Apostelgeschichte 17,28)
 *
 *      Nach 5.Mose 19,15 ist die Sache erst gltig, wenn
 *      - der Algorithmus verstndlich erklrt wurde, oder
 *      - ein oder zwei weitere Verfahren dasselbe Ergebnis erzeugen.
 *      Die "black box"-Mentalitt aus Amerika steht dabei im Gegensatz
 *      zu 3.Mose 19,14, wonach dem Blinden kein Hindernis gelegt werden
 *      darf.
 *
 *      Auer dem Ergebnis wird auch die bentigte Rechenzeit ausgegeben.
 */

#include        <SOBS.Kpf>
#include        <StdEA.Kpf>
#include        <StdBib.Kpf>


/**********************************************************/
  Nichts         Hauptteil( Ganzzahl     ParameterAnzahl ,
                            Buchstabe   *Parameter[ ] )
/**********************************************************/
{
  Buchstabe              ErgebnisDateiName[ ] = "Conway.csv" ,
                         PrimzahlDateiName[ ] = "Conway.dmp" ,
                        *AusgabeFormat[ ] = {
                           "%10lu.Ergebnis = %10lu: %10lu Schleifen\n" ,
                           "%10lu.Ergebnis = %10lu: %10lu Schleife\n"
                         } ;
  NatuerlicheZahl        pb ;
  GrosseNatuerlicheZahl  z2  , z3  , z5  , z7  , z11 ,
                         z13 , z17 , z19 , z23 , z29 ,
                         s , p , pmax ;
  Datei                 *ErgebnisDatei ,
                        *PrimzahlDatei ;
  ZeitStruktur           Zeit ;

  /* Vorbereitung: */
  /*---------------*/
  Wenn( ParameterAnzahl  >  1 )
    pmax = TextZuNatuerlicherZahl( Parameter[ 1 ] , NullZeiger , 10 ) ;
  Sonst
    pmax = 0 ;
  Wenn( !pmax )
    BildAusgabe( "Syntax:\nConway [AnzahlPrimzahlen]\n\n" ) ;
  pb = PlatzBedarf( GrosseNatuerlicheZahl ) ;
  SetzenWannStrgC( 1 ) ;

  /* Primzahlen erzeugen und ausgeben: */
  /*-----------------------------------*/
  ErgebnisDatei = DateiOeffnen( ErgebnisDateiName , "w+t" ) ;
  Wenn( !ErgebnisDatei )
    BildAusgabe( "Fehler beim ffnen der Datei \"%s\".\n\7" ,
                 ErgebnisDateiName ) ;
  PrimzahlDatei = DateiOeffnen( PrimzahlDateiName , "w+b" ) ;
  Wenn( !PrimzahlDatei )
    BildAusgabe( "Fehler beim ffnen der Datei \"%s\".\n\7" ,
                 PrimzahlDateiName ) ;

    ZeitAbfragen( &Zeit ) ;
    DateiAusgabe(
      ErgebnisDatei ,
      "%02u:%02u:%02u:%02u\n" ,
      Zeit.Zeit_Stunden ,
      Zeit.Zeit_Minuten ,
      Zeit.Zeit_Sekunden ,
      Zeit.Zeit_HundertstelSekunden
    ) ;
    z2 = 1 ;
    s = z3 = z5 = z7 = z11 = z13 = z17 = z19 = z23 = z29 = 0 ;

    ZaehlSchleife( p = 1 ; p  <=  pmax ; p++ ) {
      Schleife {
        s++ ;
        Wenn( z7   &&   z13 ) {
          z7-- ;
          z13-- ;
          z17++ ;
        }
        Sonst {
          Wenn( z5   &&   z17 ) {
            z5-- ;
            z17-- ;
            z2++ ;
            z3++ ;
            z13++ ;
          }
          Sonst {
            Wenn( z3   &&   z17 ) {
              z3-- ;
              z17-- ;
              z19++ ;
            }
            Sonst {
              Wenn( z2   &&   z19 ) {
                z2-- ;
                z19-- ;
                z23++ ;
              }
              Sonst {
                Wenn( z3   &&   z11 ) {
                  z3-- ;
                  z11-- ;
                  z29++ ;
                }
                Sonst {
                  Wenn( z29 ) {
                    z29-- ;
                    z7++ ;
                    z11++ ;
                  }
                  Sonst {
                    Wenn( z23 ) {
                      z23-- ;
                      z5++ ;
                      z19++ ;
                    }
                    Sonst {
                      Wenn( z19 ) {
                        z19-- ;
                        z7++ ;
                        z11++ ;
                      }
                      Sonst {
                        Wenn( z17 )
                          z17-- ;
                        Sonst {
                          Wenn( z13 ) {
                            z13-- ;
                            z11++ ;
                          }
                          Sonst {
                            Wenn( z11 ) {
                              z11-- ;
                              z13++ ;
                            }
                            Sonst {
                              Wenn( z2   &&   z7 ) {
                                z2-- ;
                                z7-- ;
                                z3++ ;
                                z5++ ;
                              }
                              Sonst {
                                Wenn( z2 ) {
                                  z2-- ;
                                  z3++ ;
                                  z5++ ;
                                }
                                Sonst {
                                  z5++ ;
                                  z11++ ;
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      } Solange( z3    ||   z5    ||   z7    ||    z11   ||
                 z13   ||   z17   ||   z19   ||    z23   ||   z29 ) ;
      BildAusgabe( AusgabeFormat[ s  ==  1 ? 1 : 0 ] , p , z2 , s ) ;
      InDateiSchreiben( &z2 , pb , 1 , PrimzahlDatei ) ;
      DateiAusgabe( ErgebnisDatei , "%lu,%lu,%lu\n" , p , z2 , s ) ;
      s = 0 ;
    }
    ZeitAbfragen( &Zeit ) ;
    DateiAusgabe(
      ErgebnisDatei ,
      "%02u:%02u:%02u:%02u\n" ,
      Zeit.Zeit_Stunden ,
      Zeit.Zeit_Minuten ,
      Zeit.Zeit_Sekunden ,
      Zeit.Zeit_HundertstelSekunden
    ) ;
  AlleDateienSchliessen( ) ;
}
/*__ Ende von Hauptteil __*/

/*
 * Ende von "Conway.c"
 */
